; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 13 - Line Following Car
; Link: https://www.circuitbread.com/tutorials/line-following-car-part-13-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
PSA         EQU 0003h
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h

PS0         EQU 0000h
PS1         EQU 0001h
PS2         EQU 0002h

;----- STATUS Bits -----------------------------------------------------
Z           EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)

i       EQU 10    ;Delay register 1
j       EQU 11    ;Delay register 2
servo1  EQU 12    ;Servo1 pulse width
servo2  EQU 13    ;Servo2 pulse width


; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~(1<<T0CS)
    OPTION              ;Enable GP2
    MOVLW ~((1 << GPIO_GP0_POSITION) | (1 << GPIO_GP2_POSITION))    
    TRIS GPIO           ;Set GP0 and GP2 as outputs
LOOP:
    BTFSC GPIO, GPIO_GP1_POSITION   ;Check if GP1 is low (sensor is above the line)
    GOTO MOVE_RIGHT     ;If not then go to the MOVE_RIGHT label
MOVE_LEFT:              ;Move the robot to the left
    MOVLW 200           ;Load the delay value for the servo 1
    MOVWF servo1
    MOVLW 215           ;Load the delay value for the servo 2
    MOVWF servo2
    GOTO CONTROL_SERVO  ;Go to the CONTROL_SERVO label
MOVE_RIGHT:             ;Move robot to the right
    MOVLW 206           ;Load the delay value for the servo 1
    MOVWF servo1
    MOVLW 220           ;Load the delay value for the servo 2
    MOVWF servo2
CONTROL_SERVO:          ;Control the servo 1
    BSF GPIO, GPIO_GP2_POSITION   	 ;Set GP2 high
    MOVLW 2             ;Load 2 into the second delay register 'j'
    MOVWF j
    MOVF servo1, W      ;Copy the value of the servo1 into the W
    CALL DELAY          ;and call the delay
    BCF GPIO, GPIO_GP2_POSITION     ;Then seth GP2 low
    NOP                 ;One cycle delay before the BSF instruction
SERVO_2:                ;Control the servo 2
    BSF GPIO, GPIO_GP0_POSITION    ;Set GP0 high
    MOVLW 2             ;Load 2 into the second delay register 'j'
    MOVWF j
    MOVF servo2, W      ;Copy the value of the servo2 into the W
    CALL DELAY          ;and call the delay
    BCF GPIO, GPIO_GP0_POSITION ;Then seth GP0 low
PAUSE:                  ;Pause between the pulses
    MOVLW 25            ;Load 25 into the second delay register 'j'
    MOVWF j
    CALL DELAY          ;and call the delay
	GOTO LOOP       ;Return to the 'LOOP' label

DELAY:                  ;Start DELAY subroutine here
    MOVWF i             ;Load the W value into the 'i' register
DELAY_LOOP:
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    DECFSZ j, F         ;Decrement j and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    RETLW 0             ;Else return from the subroutine

    END INIT        ; Program entry point.